#!/usr/bin/perl
########################################################
#	Bob yeaw
#	started: 8/14/02
#    last update: 1/3/03
#	widevine Technologies
#	vlsadmin.pl
#	Used to control a videolan server
#	NOTE: this requires Net::Telnet, available on CPAN
#	at http://cpan.org/authors/id/J/JR/JROGERS/
########################################################

use Net::Telnet ();

########################################################
#	User Variables:

$host = '10.100.100.0';	# videolan server ip
$port = 9999;			# videolan server admin port
$default_user = 'bozo';  # used if -u username is not supplied
$default_pass = 'bozo';  # used if -w password is not supplied
$default_input = 'local1';    # used if -i input is not supplied
$debug = 0;              # set to 1 to print debug info

########################################################
# 	main body

if ($debug) { warn "printing debug info.\n"; }
if ($debug) {
     for ($i=0; $i <= $#ARGV; $i++)
     { warn "Argument $i is $ARGV[$i].\n"; }
}

#take the user input
if($#ARGV < 0){
     &vls_help();
     die "No user input entered.\n";
}
# parse through passed in switches
for ($i=1; $i <= $#ARGV; $i++){
     SWITCH:   {
          if ($ARGV[$i] =~ /^-p/i){$progID = $ARGV[$i+1]; last SWITCH; }
          if ($ARGV[$i] =~ /^-c/i){$channel = $ARGV[$i+1]; last SWITCH; }
          if ($ARGV[$i] =~ /^-i/i){$input = $ARGV[$i+1]; last SWITCH; }
          if ($ARGV[$i] =~ /^-l/i){$loop = 1; last SWITCH; }
          if ($ARGV[$i] =~ /^-u/i){$user = $ARGV[$i+1]; last SWITCH; }
          if ($ARGV[$i] =~ /^-w/i){$pass = $ARGV[$i+1]; last SWITCH; }
     }
}
# use some defaults if no others are supplied
if ($user eq ''){
     $user=$default_user;
     if ($debug) {warn "Using default user: $default_user\n";}
}
if ($pass eq ''){
     $pass=$default_pass;
     if ($debug) {warn "Using default password: $default_pass\n";}
}
if ($input eq ''){
     $input=$default_input;
     if ($debug) {warn "Using default input: $default_input\n";}
}

if ($debug) {
     warn "progID=$progID\nchannel=$channel\ninput=$input\nloop=$loop\nuser=$user\npass=$pass\n";
}

# determine what VLS command is to be issued
SWITCH:   {
     if ($ARGV[0] =~ /^help/i || $ARGV[0] =~ /^-\?/){$cmd = 'help'; last SWITCH; }
     if ($ARGV[0] =~ /^start/i){$cmd = 'start'; last SWITCH; }
     if ($ARGV[0] =~ /^stop/i){$cmd = 'stop'; last SWITCH; }
     if ($ARGV[0] =~ /^suspend/i){$cmd = 'suspend'; last SWITCH; }
     if ($ARGV[0] =~ /^resume/i){$cmd = 'resume'; last SWITCH; }
     if ($ARGV[0] =~ /^shutdown/i){$cmd = 'shutdown'; last SWITCH; }
#     if ($ARGV[0] =~ /^show/i){$cmd = $ARGV[0]; last SWITCH; }
#     if ($ARGV[0] =~ /^config/i){$cmd = $ARGV[0]; last SWITCH; }
#     if ($ARGV[0] =~ /^program/i){$cmd = $ARGV[0]; last SWITCH; }
     if ($ARGV[0] =~ /^browse/i){$cmd = 'browse'; last SWITCH; }
#     if ($ARGV[0] =~ /^channel/i){$cmd = $ARGV[0]; last SWITCH; }
}

if ($debug) { warn "cmd=$cmd\n"; }

if ($cmd eq ''){
     &vls_help();
     die "No command specified.\n";
}

if ( $cmd eq 'help'){ &vls_help(); } # help command doesn't require a connection
else{
     $t = new Net::Telnet (	Timeout => 10,
				Host => $host,
				Port => $port,
                    Prompt => '/\@vls>/' );

     &vls_login("$user","$pass");

     #$t->prompt('/\@vls>/');  #    must do this after login. login uses different prompt

     if ($cmd eq 'start'){ &vls_start("$progID","$channel","$input","$loop"); &vls_logout(); }
     if ($cmd eq 'stop'){ &vls_stop("$channel"); &vls_logout(); }
     if ($cmd eq 'suspend'){ &vls_suspend("$channel"); &vls_logout(); }
     if ($cmd eq 'resume'){ &vls_resume("$channel"); &vls_logout(); }
     if ($cmd eq 'browse'){ &vls_browse("$input"); &vls_logout(); }
     if ($cmd eq 'shutdown'){ &vls_shutdown(); }
}

exit;

#######################################################
#	functions
#################
#	Function: 	vls_login
#	purpose: 	login to vls with supplied username and password.
#	Arguments:	username and password and scalars
#	preconditions:	Telnet connection object $t exists

sub vls_login{

  $user= $_[0];
  $pass= $_[1];

  $t->open();
  $t->waitfor('/Login:/');
  $t->print($user);
  $t->waitfor('/Password:/');
  $t->print($pass);
  $t->print("");
  # a pattern match timeout error here means the login is bad.
  $t->waitfor('/\@vls>/');
  print "Login Successful!\n";
}

#################
#	Function: 	vls_logout
#	purpose: 	logout of vls
#	Arguments:	None
#	preconditions:	Telnet connection object $t exists; login() has been called.

sub vls_logout{
  $t->print("");
  $t->waitfor('/\@vls>/');
  $t->print('logout');
  #add check here.
  print "Logout Successful!\n";
}

#################
#	Function: 	vls_start
#	purpose: 	start program from videolan server
#	Arguments:	program channel input and optional loop as scalars
#	preconditions:	Telnet connection object $t exists; login() has been called.

sub vls_start{
  $progID = $_[0];
  $channel = $_[1];
  $input = $_[2];
  $loop = $_[3];
  
  $t->print("");
  $t->waitfor('/\@vls>/');
  if ($loop){	#loopInfo: Unknown pgrm
     if ($debug) { warn("start $progID $channel $input --loop\n"); }
     $t->cmd("start $progID $channel $input --loop");
  }else{		#No loop
     if ($debug) { warn("start $progID $channel $input\n"); }
     $t->cmd("start $progID $channel $input");
  }
  ($lines) = $t->waitfor('/\@vls>/');
  if ($debug) { print "$lines\n"; }
  
  if ($lines =~ /Program $progID started/)
     {print "$progID started on $channel.\n";}
  if ($lines =~ /Channel is busy/)
     {warn "Channel $channel is busy.\n";}
  if ($lines =~ /Unknown pgrm/)
     {warn "Invalid program $progID.\n";}
  if ($lines =~ /Unknown channel/)
     {warn "Invalid channel $channel.\n";}
  if ($lines =~ /No such file or directory/)
     {die "Problem with program $progID: No such file or directory.\n";}
}

#################
#	Function: 	vls_stop
#	purpose: 	stop program running on videolan server
#	Arguments:	channel as scalar
#	preconditions:	Telnet connection object $t exists; login() has been called;
#			start_prog has been called.

sub vls_stop
{
  $channel = $_[0];
  
  $t->print("");
  $t->waitfor('/\@vls>/');
  $t->cmd("stop $channel");
  ($lines) = $t->waitfor('/\@vls>/');
  if ($debug) {print "$lines\n";}
  if ($lines =~ /stopped/)
     { print "Program stopped on channel $channel.\n";}
  if ($lines =~ /Nothing broadcasted to channel/)
     { warn "Nothing broadcasted to channel $channel.\n"; }
}

#################
#	Function: 	vls_suspend
#	purpose: 	suspend program running on videolan server
#	Arguments:	channel as scalar
#	preconditions:	Telnet connection object $t exists; login() has been called;
#			start_prog has been called.

sub vls_suspend
{
  $channel = $_[0];

  $t->print("");
  $t->waitfor('/\@vls>/');
  $t->cmd("suspend $channel");
  ($lines) = $t->waitfor('/\@vls>/');

  if ($debug) {print "$lines\n";}
  if ($lines =~ /suspended/)
     { print "Program suspended on channel $channel.\n";}
  if ($lines =~ /wasn't running/)
     { print "Program wasn't running on channel $channel\n";}
  if ($lines =~ /Nothing broadcasted to channel/)
     { warn "Nothing broadcasted to channel $channel.\n"; }
}

#################
#	Function: 	vls_resume
#	purpose: 	resume suspended program running on videolan server
#	Arguments:	channel as scalar
#	preconditions:	Telnet connection object $t exists; login() has been called;
#			start_prog has been called, channel has been suspended.

sub vls_resume
{
  $channel = $_[0];

  $t->print("");
  $t->waitfor('/\@vls>/');
  $t->cmd("resume $channel");
  ($lines) = $t->waitfor('/\@vls>/');

  if ($debug) {print "$lines\n";}
  if ($lines =~ /resumed/)
     { print "Program resumed on channel $channel.\n";}
  if ($lines =~ /isn't suspended/)
     { print "Program wasn't suspended on channel $channel\n";}
  if ($lines =~ /Nothing broadcasted to channel/)
     { warn "Nothing broadcasted to channel $channel.\n"; }
}

#################
#	Function: 	vls_browse
#	purpose:  returns the Videolan server program list
#	Arguments:   input (optional)
#	preconditions:	Telnet connection object $t exists; login() has been called;

sub vls_browse
{
  $input = $_[0];
  
  $t->print("");
  $t->waitfor('/\@vls>/');
  $t->cmd("browse $input");
  ($lines) = $t->waitfor('/\@vls>/');
  print "$lines\n";
}

#################
#	Function: 	vls_shutdown
#	purpose: 	shutdown videolan server
#	Arguments:   none
#	preconditions:	Telnet connection object $t exists; login() has been called;

sub vls_shutdown
{
  $t->print("");
  $t->waitfor('/\@vls>/');
  $t->cmd("shutdown");
  ($lines) = $t->waitfor('/\@vls>/');

  if ($debug) {print "$lines\n";}
  if ($lines =~ /Shutdowning the server/)
     { print "Shutdowning the server.\n";}
}

#################
#	Function:   vls_help
#	purpose: 	show videolan server help info
#	Arguments:   none
#	preconditions:	none

sub vls_help
{
print ("
Usage:
 perl vlsadmin cmd [-p progID] [-c channel] [-i input] [-l] [-u user] [-w pw]\n
 cmd
     help: your looking at it
     vlsadmin help or -?
     
     browse: gives the program list
     vlsadmin browse [-i input]

     start: launches a program
     vlsadmin start -p progID -c channel [-i input] [-l] [-u user] [-w pw]

     stop: stops a program
     vlsadmin stop -c channel [-u user] [-w pw]

     suspend: suspends streaming
     vlsadmin suspend -c channel [-u user] [-w pw]

     resume: resumes streaming
     vlsadmin resume -c channel [-u user] [-w pw]

     shutdown: stops the server
     vlsadmin shutdown [-u user] [-w pw]\n

 -p   Program ID is required with cmds: start
 -c   Channel is required with cmds: start, stop, suspend, resume
 -i   Input is required with cmds: start
 -l   Loop is optional with cmds: start
 -u   Username is optional; a defualt one will be used if none is supplied.
 -w   Password is optional; a defualt one will be used if none is supplied.
 -?   Same as the help command.
");
}
#########################################################
#	END

